//Author: Gerben de Keijzer
//Licence: WTFPL except for some icons by Alexandre Moore that are GPL
(function(window, document, undefined)
{
	var mpdApiUrl = '';
	var fileServerUrl = false;

	/*
	 * helper functions
	 */
	document.head = document.head || document.getElementsByTagName('head')[0];
	document.body = document.body || document.getElementsByTagName('body')[0];
	var log = function(str)
	{
		if( window.console && console.log )
		{
			console.log(str);
		}
	};
	var trim = function (s)
	{
		return s.replace(/^\s+|\s+$/g, '');
	};


	/*
	 * Volume Control
	 */
	var volumeControl = (function(){
		var vc = {}
		//private
		var volumeStep = 10;
		var currentVolume = 0;
		var muted = false;

		var volumeBar, volumeButton;
		var update = function(value)
		{
			if( !volumeBar && !volumeButton ) {
				volumeBar = document.getElementById('volumebarfill');
				volumeButton = document.getElementById('volume');
			}
			volumeBar.style.height = currentVolume+'%';
			volumeBar.className = muted ? 'mute' :'';
			volumeButton.className = muted ? 'mute' : currentVolume<21 ? 'low' : currentVolume<61 ? 'medium' : 'high';
		};

		vc.getVolume = function()
		{
			return muted ? 0 : currentVolume;
		}

		vc.setVolume = function(value)
		{
			if(muted&&value==0)return;//server updates value to 0
			currentVolume = Math.max(Math.min(1*value, 100),0);// 0..100
			muted = (currentVolume==0)
			update();
			return currentVolume;
		};
		vc.volumeUp = function()
		{
			return vc.setVolume(currentVolume+volumeStep)
		};
		vc.volumeDown = function()
		{
			return vc.setVolume(currentVolume-volumeStep)
		};
		vc.toggleMute = function()
		{
			muted = !muted;
			if( !muted )
				currentVolume = Math.max(volumeStep, currentVolume);//if muted by lowering to 0, unmute would do nothing
			update()
			return muted ? 0 : currentVolume;
		}
		vc.mute = function()
		{
			muted = true;
			update();
			return 0;
		};

		return vc;
		})();


	/*
	 * display song and track info
	 */


	var currentIndex = 0;
	var currentPlaylistVersion = 0;
	var playlistChanged = true;


	var songStartTime=0, songDuration=0, timeIntervalId;
	var updateTime = function(times)
	{
		var secondsPlayed, totalSeconds, now=new Date();
		if( times===undefined )
		{
			secondsPlayed = Math.round((now - songStartTime)/1000);
			totalSeconds = songDuration;
			if( totalSeconds>0 )
				secondsPlayed = Math.min(secondsPlayed, totalSeconds);//don't extrapolate beyond total
				//todo: request status update if secondsPlayed > totalSeconds 
		}
		else
		{
			var timeparts = times.split(':');
			secondsPlayed	= 1*timeparts[0];
			totalSeconds	= 1*timeparts[1];
			songStartTime = now - 1000*secondsPlayed;
			songDuration = totalSeconds;
		}
		var timestr = Math.floor(secondsPlayed/60) + ':' + (secondsPlayed%60<10?'0':'') + secondsPlayed%60;
		if( totalSeconds>0 )
		{
			timestr += '/' + Math.floor(totalSeconds/60) + ':' + (totalSeconds%60<10?'0':'') + totalSeconds%60;
		}
		document.getElementById('tracktime').innerHTML = timestr;
		document.getElementById('progressbar').style.width = (totalSeconds ? Math.round(1000*secondsPlayed/totalSeconds)/10 : 0)+'%';
	};
	setInterval(function(){if(document.body.className.indexOf('play')>=0){updateTime();}}, 1000);


	var displayStatusUpdate = function(properties)
	{
		// highlight buttons
		var className = properties['state'];
		if( properties['time'] && properties['time'].indexOf(':0')>-1 )
			className += ' isstream';
		className += (properties['repeat']==='1') ? ' repeat' : ' norepeat';
		className += (properties['random']==='1') ? ' random' : ' norandom';
		className += ' p'+properties['song'];
		document.body.className = className;


		// volume
		volumeControl.setVolume(properties['volume']);

		// songindex / playlistlength
		document.getElementById('trackindex').innerHTML = (1+1*properties['song'])+'/'+properties['playlistlength'];


		// time: <int elapsed>:<time total> (of current playing/paused song)
		if( !properties['time'] )
			properties['time']='0:0';
		updateTime(properties['time'], properties['state']==='play');
	};


	var displayCurrentSongUpdate = function(properties)
	{
		var trackname = properties['Name'] || '&nbsp;';
		var tracktitle = properties['Title'] || '&nbsp;';
		document.getElementById('trackname').innerHTML = cleanTrackName(trackname);
		document.getElementById('tracktitle').innerHTML = tracktitle;
	};

	var cleanTrackName = function(str)
	{
		str = str.replace('- D I G I T A L L Y - I M P O R T E D','');
		if( str.indexOf('http://')==0 && str.indexOf('#')>0 )
			str = str.substr(str.indexOf('#')+1);
		return str;
	};

	var displayPlaylistInfoUpdate = function(playlist)
	{
		var pl = document.getElementById('playlist');
		var html = '<ul>';
		for(var i=0,l=playlist.length; i<l; i++)
		{
			var name = playlist[i]['Name'] || playlist[i]['file'];
			name = cleanTrackName(name);
			html += '<li class="i'+i+'"><a class="plitem" date-index="'+i+'" href="#">'+name+'</a></li>';
		}
		html +='</ul>';
		pl.innerHTML = html;
	};



	/*
	 * retreiving mpd info
	 */

	var parseMpdProperties = function(str)
	{
		var parts = str.split('\n');
		var part, index, key, val;
		var properties = {};
		for( var i=0, l=parts.length; i<l; i++ )
		{
			part = parts[i];
			index = part.indexOf(':');
			if( index>0 )
			{
				key = part.substr(0, index);
				val = trim(part.substr(index+1));
				properties[key]=val;
			}
		}
		return properties;
	};


	/* update status (time, volume, trackindex, repeat, etc.) */
	var xhr1 = false;
	var requestStatusUpdate = function()
	{
		if( xhr1 && (xhr1.readyState>1 && xhr1.readyState<4) )
			return false;
		if( !xhr1 )
		{
			if( window.XDomainRequest )
				xhr1 = new XDomainRequest();
			else
				xhr1 = new XMLHttpRequest();

		}
		//if( xhr1.readyState>1 && xhr1.readyState<4 )
		//	xhr1.abort();
		xhr1.open('POST', mpdApiUrl, true);
		xhr1.onreadystatechange = xhr1Readystatechange;
		xhr1.send('status');
	};
	var xhr1Readystatechange = function()
	{
		if( this.readyState==4 && this.responseText )
		{
			if( location.href.indexOf('?')>0)//debug
				document.getElementById('status').innerHTML = this.responseText;
			var properties = parseMpdProperties(this.responseText);
			//log(properties);
			displayStatusUpdate(properties);
			if( currentPlaylistVersion != properties['playlist'] )
			{
				playlistChanged = true;
				currentPlaylistVersion = properties['playlist'];
				log('playlist changed');
			}

			// also update currentsong (name+title)
			requestCurrentSongUpdate();
		}
	};


	/* update currentsong (name, title, (file+pos+id)) */
	var xhr2 = false;
	var requestCurrentSongUpdate = function()
	{
		if( xhr2 && (xhr2.readyState>1 && xhr2.readyState<4) )
			return false;
		if( !xhr2 )
		{
			if( window.XDomainRequest )
				xhr2 = new XDomainRequest();
			else
				xhr2 = new XMLHttpRequest();

		}
		//if( xhr1.readyState>1 && xhr1.readyState<4 )
		//	xhr1.abort();
		xhr2.open('POST', mpdApiUrl, true);
		xhr2.onreadystatechange = xhr2Readystatechange;
		xhr2.send('currentsong');
	};

	var xhr2Readystatechange = function()
	{
		if( this.readyState==4 && this.responseText )
		{
			var properties = parseMpdProperties(this.responseText);
			//log(properties);
			displayCurrentSongUpdate(properties);

			// also update playlistinfo
			if( playlistChanged )
				requestPlaylistInfoUpdate();
			//todo: only if playlistid has changed
			//todo: update pos if pos changed
		}
	};


	/* update playlistinfo */
	var xhr3 = false;
	var requestPlaylistInfoUpdate = function()
	{
		playlistChanged = false;
		if( xhr3 && (xhr3.readyState>1 && xhr3.readyState<4) )
			return false;
		if( !xhr3 )
		{
			if( window.XDomainRequest )
				xhr3 = new XDomainRequest();
			else
				xhr3 = new XMLHttpRequest();

		}
		//if( xhr1.readyState>1 && xhr1.readyState<4 )
		//	xhr1.abort();
		xhr3.open('POST', mpdApiUrl, true);
		xhr3.onreadystatechange = xhr3Readystatechange;
		xhr3.send('playlistinfo');
	};

	var xhr3Readystatechange = function()
	{
		if( this.readyState==4 && this.responseText )
		{
			var playlist = parseMpdPlaylist(this.responseText);
			//log(playlist);
			displayPlaylistInfoUpdate(playlist);
		}
	};
	var parseMpdPlaylist = function(str)
	{
		var playlist = [];

		var parts = str.split('\n');
		var part, index, key, val;
		var properties = {};
		for( var i=0, l=parts.length; i<l; i++ )
		{
			part = parts[i];
			index = part.indexOf(':');
			if( index>0 )
			{
				key = part.substr(0, index);
				val = trim(part.substr(index+1));
				properties[key]=val;
				if( key=='Id' )
				{
					// save properties of the playlist item
					playlist.push(properties);
					properties = {};
				}
			}
		}
		return playlist;
	};



	/* send commands (play, stop, etc.) */
	var xhr9 = false;
	var sendCommand = function(cmd)
	{
		log('command: '+cmd);
		if( !xhr9 || (xhr9.readyState>0 && xhr9.readyState<4) ) //busy or not yet initialized
		{
			if( window.XDomainRequest )
				xhr9 = new XDomainRequest();
			else
				xhr9 = new XMLHttpRequest();
			xhr9.onreadystatechange = function()
			{
				if( this.readyState==4 && this.responseText )
				{
					// update all info
					requestStatusUpdate();
				}
			};
		}
		xhr9.open('POST', mpdApiUrl, true);
		xhr9.send(cmd);
	};
	
	var volumeTimeoutId;
	var showVolume = function(){
		var volpopup = document.getElementById('volumepopup');
		volpopup.style.display='';
		clearTimeout(volumeTimeoutId);
		volumeTimeoutId = setTimeout(function(){volpopup.style.display='none'}, 4e3);
	}



	/*
	 * event handling of all buttons
	 */

	var handleControlClick = function(e)
	{
		if( window.event ) e=window.event;

		var target = e.target;
		while( target && target!=document.body && target.nodeName!='A' )
			target = target.parentNode;

		if( target.nodeName=='A' )
		{
			var preventDefault = true;
			switch ( target.id || target.className )
			{
				case 'prev':
					sendCommand('previous')
					break;
				case 'stop':
					sendCommand('stop')
					break;
				case 'play':
					sendCommand('play')
					break;
				case 'pause':
					sendCommand('pause')
					break;
				case 'next':
					sendCommand('next')
					break;
				case 'repeat':
					sendCommand('repeat 1')
					break;
				case 'norepeat':
					sendCommand('repeat 0')
					break;
				case 'volume':
					var volpopup = document.getElementById('volumepopup');
					if( volpopup.style.display )
						showVolume();
					else
						volpopup.style.display='none';
					break;
				case 'volumeup':
					var vol = volumeControl.volumeUp();
					sendCommand('setvol '+ vol);
					showVolume();
					break;
				case 'volumedown':
					var vol = volumeControl.volumeDown();
					sendCommand('setvol '+ vol);
					showVolume();
					break;
				case 'togglemute':
					var vol = volumeControl.toggleMute();
					sendCommand('setvol '+ vol);
					showVolume();
					break;
				case 'plitem':
					var index = target.getAttribute('date-index');
					document.body.className = document.body.className.replace(' p', ' p'+index+' oldp');
					sendCommand('play '+index);
					break;
				default:
					preventDefault = false;//follow normal link
					alert(target.id || target.className);
			}
			if( preventDefault )
				e.preventDefault();
		}

	}


	var init = function()
	{
		document.title = 'MPClient';//+' version 0.3';
		//get url for the mpd api
		var links = document.getElementsByTagName('link');
		for( var i=0,l=links.length; i<l; i++ )
		{
			if( links[i].rel=='mpd' )
			{
				mpdApiUrl = links[i].href;//firefox automatically adds the domainname in front; but doesn't matter since relative urls work also (in this case)
				break;
			}
		}
		
		//get base url (for css), relative to current js file
		fileServerUrl = document.getElementsByTagName('script')[0].src;
		fileServerUrl = fileServerUrl.substr(0, fileServerUrl.lastIndexOf('/')+1);
		
		//add stylesheet
		var newSS=document.createElement('link');
		newSS.rel='stylesheet';
		newSS.href=fileServerUrl+'mpc.css';//+'?'+Math.random();
		document.head.appendChild(newSS);
		//add favicon
		var link = document.createElement('link');
		link.type = 'image/x-icon';
		link.rel = 'shortcut icon';
		link.href = fileServerUrl+'img/favicon.png';
		document.head.appendChild(link);
		//add apple touch icon
		var link=document.createElement('link');
		link.rel='apple-touch-icon-precomposed';
		link.href=fileServerUrl+'img/touchicon.png';
		document.head.appendChild(link);
		//<meta name="apple-mobile-web-app-capable" content="yes">
		var meta=document.createElement('meta');
		meta.name='apple-mobile-web-app-capable';
		meta.content='yes';
		document.head.appendChild(meta);
		//<meta name="apple-mobile-web-app-status-bar-style" content="black" />
		var meta=document.createElement('meta');
		meta.name='apple-mobile-web-app-status-bar-style';
		meta.content='black';
		document.head.appendChild(meta);
		
		//create body html
		document.body.innerHTML = '\
<div id="trackinfo">\
	<div id="trackname">Loading...</div>\
	<div id="tracktitle">&nbsp;</div>\
	<div id="progress">\
		<div id="time"></div><div id="progressbar"></div><div id="totaltime"></div></div>\
	</div>\
\
<div id="controls">\
\
<div id="playback">\
<a href="#" id="prev">previous</a> \
<a href="#" id="stop">stop</a> \
<a href="#" id="play">play</a> \
<a href="#" id="pause">pause</a> \
<a href="#" id="next">next</a> \
<a href="#" id="repeat">repeat</a> \
<a href="#" id="norepeat">norepeat</a> \
<a href="#" id="volume">volume</a> \
</div>\
\
<div id="volumepopup">\
<a href="#" id="togglemute">toggle mute</a><a href="#" id="volumedown">volume down</a><span id="volumebar"><span id="volumebarfill"></span></span> <a href="#" id="volumeup">volume up</a>\
</div>\
<div id="trackindex"></div><div id="tracktime"></div>\
\
<div id="playlist"></div>\
\
</div>\
\
<pre id="status"></pre>';

		requestStatusUpdate();
		setInterval(requestStatusUpdate, 5000);

		//if( window.opera )
		//	setTimeout(function(){location.href='#controls';}, 1000);

		document.getElementById('controls').addEventListener('click',handleControlClick,true);
		
		//hide volume controls
		var volpopup = document.getElementById('volumepopup');
		volpopup.style.display='none';
		
		//detect touch
		try {
			document.createEvent("TouchEvent");
			document.documentElement.className='touch';
		} catch (e) {}
		
		
	}
	//window.onload = init;
	init();

})(this, this.document);